%Download and prepare dataset
baseDownloadURL = 'https://www.mrt.kit.edu/z/publ/download/velodyneslam/data/scenario1.zip';
dataFolder      = fullfile(tempdir, 'kit_velodyneslam_data_scenario1', filesep);
options         = weboptions('Timeout', Inf);
zipFileName  = dataFolder + "scenario1.zip";
pointCloudFilePattern = fullfile(dataFolder, 'scenario1', 'scan*.png');
numExpectedFiles = 2513;
folderExists = exist(dataFolder, 'dir');
if ~folderExists
    mkdir(dataFolder);
    disp('Downloading scenario1.zip (153 MB) ...')
    websave(zipFileName, baseDownloadURL, options);
    unzip(zipFileName, dataFolder);
elseif folderExists && numel(dir(pointCloudFilePattern)) < numExpectedFiles
    disp('Downloading scenario1.zip (153 MB) ...') 
    websave(zipFileName, baseDownloadURL, options);
    unzip(zipFileName, dataFolder)    
end

datasetTable = helperReadDataset(dataFolder, pointCloudFilePattern);

%Tabelle contententi le point cloud e corrispondenti dati inerziali
pointCloudTable = datasetTable(:, 1);
insDataTable    = datasetTable(:, 2:end);

%Ad esempio, recupero e mostro la prima point cloud...
ptCloud = helperReadPointCloudFromFile(pointCloudTable.PointCloudFileName{1});
disp(ptCloud)

%e la pose misurata tramite l'unità inerziale
disp(insDataTable(1, :))




%Mostro lo stream delle point cloud 
xlimits = [-45 45];
ylimits = [-45 45];
zlimits = [-10 20];

lidarPlayer = pcplayer(xlimits, ylimits, zlimits);
title(lidarPlayer.Axes, 'Point Clouds')

skipFrames  = 2;
numFrames   = height(pointCloudTable);
for n = 1 : skipFrames : numFrames
    fileName = pointCloudTable.PointCloudFileName{n};
    ptCloud = helperReadPointCloudFromFile(fileName);
    view(lidarPlayer, ptCloud);    
    pause(0.01)
end

%Iniziamo a costruire una mappa fatta di sole registrazioni + IMU


hide(lidarPlayer)
rng(0);

%Un view set ci permette di visualizzare facilmente il risultato. E'
%simile a un grafo. E' composto da View (nodi), contenenti ciascuno una
%point cloud e la relativa pose. Le view sono connesse da Connection
%(archi), contenenti la trasformazione relativa tra due view.
%Si aggiunge una view con: addView(vSet, view_id, pose, "PointCloud", ptcloud);
%Si aggiunge una connection con: addConnection(vSet, first_id, second_id, transform);
vSet = pcviewset;

hFigBefore = figure('Name', 'View Set Display');
hAxBefore = axes(hFigBefore);

% Le pose (trasformazioni assolute) e le trasformazioni relative sono di
% tipo rigid3d
absTform   = rigid3d;  
relTform   = rigid3d;

% Ora dobbiamo allineare le point cloud. 
% Consiglio: usate la funzione helperProcessPointCloud(ptCloud) 
% per rimuovere il veicolo e il piano stradale prima dell'allineamento;
% Anche sottocampionare è una buona idea
% La maggior parte degli algoritmi di registration ha bisogno di una stima
% iniziale della trasformazione, potete ottenerla dai dati IMU, tramite la
% funzione helperComputeInitialEstimateFromINS(relTform, insDataTable(n-skipFrames:n, :));
% Possiamo usare plot(vSet, "Parent", hAxBefore) per mostrare il viewSet
% In questo caso dobbiamo rilevare anche le loop closure usando ScanContext
% loopDetector = scanContextLoopDetector
% Per calcolare il descrittore di una point cloud e aggiungerlo al
% loopDetector, useremo: 
% descriptor = scanContextDescriptor(ptCloud)
% addDescriptor(loopDetector, view_id, descriptor)
% Per rilevare un loop possiamo usare:
% loopViewId = detectLoop(loopDetector)
%
% Per stimare le pose, dobbiamo convertire il viewSet in un pose graph
% G = createPoseGraph(vSet);
% che ottimizzeremo con 
% optimG = optimizePoseGraph(G, 'g2o-levenberg-marquardt');
% e ritrasformeremo in un viewSet, per visualizzare il risultato
% vSetOptim = updateView(vSet, optimG.Nodes);


    
%TODO: costruire il viewset



%Costruiamo la mappa a partire dal viewSet
ptClouds = vSet.Views.PointCloud;
absPoses = vSet.Views.AbsolutePose;
mapGridSize = 0.2;
ptCloudMap = pcalign(ptClouds, absPoses, mapGridSize);


%Visualizziamo il risultato
hFigAfter = figure('Name', 'View Set Display (before optimization)');
hAxAfter = axes(hFigAfter);
pcshow(ptCloudMap, 'Parent', hAxAfter);
hold on
plot(vSetOptim, 'Parent', hAxAfter);
helperMakeFigurePublishFriendly(hFigAfter);
